local ITEM_HEIGHT = 16
local ITEM_TEXT_WIDTH = 270 
local BUTTON_TEXTURE_FNAME = [[Interface\AddOns\AltsMail\textures\statusbar\smooth]]
local list = {}


local function CreateNode(idx, parent)
	local b = CreateFrame("Button", "AltsMailList_ItemButton"..idx, parent, "AltsMailList_ItemButtonTemplate")
	b:SetWidth(parent:GetWidth() - 45)
	b:SetHeight(ITEM_HEIGHT - 2)
	b.tag = idx
    b:SetNormalTexture(BUTTON_TEXTURE_FNAME)
    b:SetHighlightTexture("Interface/Buttons/UI-Panel-Button-Highlight")
    b:SetPushedTexture("Interface/Buttons/UI-Panel-Button-Down")
	b.Text:SetTextColor(1.0, 1.0, 1.0)
	return b
end 

function AltsMailList_SetListButtons()
	local height = AltsMailList_Frame:GetHeight() - 96
	AltsMailList_Frame.sf.count = math.floor(height / ITEM_HEIGHT)
	--print('height', height, 'count', AltsMailList_Frame.sf.count)
	for i = AltsMailList_Frame.sf.count + 1, #AltsMailList_Frame.sf.buttons do
		AltsMailList_Frame.sf.buttons[i]:Hide()
	end
	for i = 1, AltsMailList_Frame.sf.count do
		if not(AltsMailList_Frame.sf.buttons[i]) then
			AltsMailList_Frame.sf.buttons[i] = CreateNode(i, AltsMailList_Frame)
			if i == 1 then
				AltsMailList_Frame.sf.buttons[i]:SetPoint("TOPLEFT", AltsMailList_Frame, "TOPLEFT", 10, -66)
			else
				AltsMailList_Frame.sf.buttons[i]:SetPoint("TOPLEFT", AltsMailList_Frame.sf.buttons[i - 1], "BOTTOMLEFT", 0, -2)
			end
		end
	end
	FauxScrollFrame_Update(AltsMailList_Frame.sf, AltsMailList_Frame.sf.count + 1, AltsMailList_Frame.sf.count, ITEM_HEIGHT)
end

local function ScrollBar_Compare(a, b)
	return a.name < b.name 
end 

local function ClearTable(t)
	for k, v in pairs(t) do
		t[k] = nil
	end
end

function AltsMailList_Update()
	ClearTable(list)
	for rname, v in pairs(AltsMail_Alts) do
		if rname ~= 'searchFlag' then
			for name, info in pairs(v) do
				local item
				if info and type(info) == 'table' then
					item = {["name"] = name..'-'..rname, ["haveMail"] = info.haveMail, ["haveAH"] = info.haveAH, ["classFileName"] = info.classFileName, ["lastLogin"] = info.lastLogin}
				else
					item = {["name"] = name..'-'..rname, ["haveMail"] = nil, ["haveAH"] = nil}
				end
				table.insert(list, item)
			end
		end
	end
	table.sort(list, ScrollBar_Compare)
	
	local line -- 1 to ts.listCount of our window to scroll
	local lineplusoffset -- an index into our data calculated from the scroll offset
	local itemcount = #list
	--print('itemcount', itemcount, 'sf.count', AltsMailList_Frame.sf.count)
	local offset = FauxScrollFrame_GetOffset(AltsMailList_Frame.sf)

	for line = 1, AltsMailList_Frame.sf.count do
		lineplusoffset = line + offset
		if (lineplusoffset <= itemcount) then
			local v = list[lineplusoffset]
			local sb = AltsMailList_Frame.sf.buttons[line]
			if (v) then
				sb.Text:SetText(v.name)
				sb.Text:SetWidth(sb:GetWidth() - 75)
				if v.haveMail then
					--sb.AvailMail:Show()
					sb.AvailMail:SetText("M")
				else
					--sb.AvailMail:Hide()
					sb.AvailMail:SetText("")
				end
				if v.haveAH then
					sb.AvailAuction:SetText("A")
				else
					sb.AvailAuction:SetText("")
				end
				
				local alpha = 0.1
				if v.lastLogin then
					local temp = date("*t", time() - v.lastLogin)
					--print(temp.day)
					if temp.day < 30 then
						alpha = 1.0 - temp.day * 0.03
					end
				end
				
				local tex = sb:GetNormalTexture()
				tex:SetVertexColor(0.5, 0.5, 0.5, alpha)
				if v.classFileName then
					local color = RAID_CLASS_COLORS[v.classFileName]
					if color then
						tex:SetVertexColor(color.r, color.g, color.b, alpha)
					end
				end
				sb:Show()
			else
				sb:Hide()
			end
		else
			AltsMailList_Frame.sf.buttons[line]:Hide()
		end
	end
	
	-- (frame, numItems, numToDisplay, valueStep, button, smallWidth, bigWidth, highlightFrame, smallHighlightWidth, bigHighlightWidth, alwaysShowScrollBar )
	FauxScrollFrame_Update(AltsMailList_Frame.sf, itemcount, AltsMailList_Frame.sf.count, ITEM_HEIGHT)	
end

function AltsMailList_VerticalScroll(self, offset)
	FauxScrollFrame_OnVerticalScroll(self, offset, ITEM_HEIGHT, AltsMailList_Update)
	--AltsMailList_Highlight(AltsMailList_Frame.selectedIdx)
end

function AltsMailList_ItemButtonClick(self)
	local idx = self.tag + FauxScrollFrame_GetOffset(AltsMailList_Frame.sf)
	--print(idx)
	if list[idx] then
		SendMailNameEditBox:SetText(list[idx].name:gsub(" ", ""))
	end
end

function AltsMailList_RemoveClick(self)
	local idx = self:GetParent().tag + FauxScrollFrame_GetOffset(AltsMailList_Frame.sf)
	--print(idx)
	if list[idx] then
		local target = list[idx].name
		--print(target)
		-- split string by '-'
		local values = {strsplit("-", target)}
		if #values > 1 then
			local name = values[1]
			-- get realm
			local rname = values[2]
			--print(rname, name)
			if AltsMail_Alts[rname] then
				AltsMail_Alts[rname][name] = nil
				--print('#AltsMail_Alts', #AltsMail_Alts)
				AltsMailList_Update()
			end
		end
	end
end

